<?php

namespace Teknisa\PLA\Modules\Parametros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use PLA\Util\Util;

class RestricoesContratuais extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
	protected $parameterBagColumns;
    protected $newCodeNT;
    protected $fileServerURL;
    protected $zhFileUploader;
    protected $util;
    protected $connection;
    protected $chaves;
    protected $environment;
    const TABLES_BY_SCREEN_CODE = array(
        "PLA03300" => array( //Limite de Per Capita por Serviço
            "TABLE_VIG"  => "VIGLIMPESERGR"
        ), 
        "PLA03200" => array( // Restrições de Produto por Grupo de Prato
            "TABLE_VIG"  => "VIGRESTPRODGR"
        ), 
        "PLA28000" => array( // Frequencia de Produtos em Grupo de Prato no Mes/Semana
            "TABLE_VIG"  => "VIGFREQPRODGRPRAT"
        ), 
        "PLA29000" => array( // Frequencia de Grupo de Pratos no Mes/Semana
            "TABLE_VIG"  => "VIGFREQGRPRATMES"
        ), 
        "PLA02900" => array( // Frequencia de Opções por grupo de prato
            "TABLE_VIG"  => "VIGFREQOPCAGR"
        ), 
        "PLA02800" => array( //Frequencia de Pratos no Mes/Semana
            "TABLE_VIG" => "VIGFREQPRATMES"
        ) 
    );

    public function __construct(
        $crudService, $parameterBag, $newCodeNT, 
        $fileServerURL, $zhFileUploader, $util, $connection, $environment
    ) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->newCodeNT         = $newCodeNT;
        $this->fileServerURL     = $fileServerURL;
        $this->zhFileUploader    = $zhFileUploader;
        $this->util              = $util;
        $this->connection        = $connection;
        $this->environment       = $environment;
        $this->chaves            = array();
    }

    //Utilities

    public function getTables() {
        return array_map(function($items){
            return $items["TABLE_VIG"];
        }, self::TABLES_BY_SCREEN_CODE);
    }
    
    public function validateDuplicatedRecords(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->util->validateDuplicatedRecords($row, $row["dataSourceName"], $row["fieldsToValidate"]);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function validateInterval(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if ($row["__is_new"]) {
                $this->validateRegisterVigencia($row);
            }
            if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                $nrvigencia = isset($row["NRVIGENCIA"]) ? $row["NRVIGENCIA"] : null;
                $this->validateIntervalOverlap($nrvigencia, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function validateRegisterVigencia($row) {
        $this->util->validateDuplicatedRecords($row, $row["dataSourceName"], $row["fieldsToValidate"]);
    }

    public function validateIntervalOverlap($vig, $start, $end, $tableName, $cdfilial, $idtiporestricao) {
        if (in_array(strtoupper($tableName), $this->getTables())) { // Preventing SQL injection
            $sql = "SELECT 1 FROM " . $tableName . 
                    " WHERE ((:DTVIGENCIA BETWEEN DTVIGENCIA AND DTVIGENCIAFIM) OR (:DTVIGENCIAFIM BETWEEN DTVIGENCIA AND DTVIGENCIAFIM))
                    AND (NRVIGENCIA <> :NRVIGENCIA OR :NRVIGENCIA IS NULL)
                    AND CDFILIAL= :CDFILIAL
                    AND IDTIPORESTRICAO = :IDTIPORESTRICAO
                    AND NRORG = :NRORG";
            $params = array(
                "NRVIGENCIA"    => $vig,
                "DTVIGENCIA"    => $start,
                "DTVIGENCIAFIM" => $end,
                "CDFILIAL"      => $cdfilial,
                "IDTIPORESTRICAO" => $idtiporestricao,
                "NRORG"         => $this->crudService->getEnvironment()->getNrOrg()
            );
            $dataSet = $this->connection->fetchAll($sql, $params);
            if (!empty($dataSet)) {
                throw new \Exception("intervalOverlap", 1);
            } 
        } else {
            throw new \Exception("tableNotFound", 1);
        }
    }

    public function getNewCode($newCodeKey, $newCodeLength = 10, $castNumber = false) {
        $nrorg = $this->crudService->getEnvironment()->getNrOrgTrab();
        $newCodeKey = strtoupper($newCodeKey);
        return $this->newCodeNT->getCodeByTableName($newCodeKey, $nrorg, 1, $newCodeLength, $castNumber);
    }

    public function buildConditions($columnNames, $values, $operators = null) {
        $conditions = array();

        foreach ($columnNames as $key => $columnName) {
            array_push($conditions, 
                array(
                    "columnName" => $columnName,
                    "operator" => is_array($operators) ? $operators[$key] : "=",
                    "value" => $values[$key]
                )
            );
        }
        return $conditions;
    }

    //Anexos
    private function setFileFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["DOCUMENTOANEXO"] !== null) {
				$row["DOCUMENTOANEXOFULLPATH"] = $this->fileServerURL.$row["DOCUMENTOANEXO"];
			}
		}
		return new DataSet("restr_contr_anexos", $rows);
	}

    public function anexosFind(DTO\Request $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $table = $conditions[1]["value"];
        $indexTable = -1;
        for($i = 0; $i < count($conditions); $i++){
            if($conditions[$i]['columnName'] == 'TABLE'){
                $indexTable = $i;
            }
        }
        if($indexTable != -1){
            array_splice($conditions, $indexTable, 1);
        }
        $dataSet = $this->crudService->getFilteredDataset($table, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $newDataSet = $this->setFileFullPath($dataSet);
        $response->addDataSet($newDataSet);
    }
    
    public function uploadFile(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->dataSourceName = $row{"TABLE"};
            foreach ($row["fileUpload"] as $value){
                if (isset($value) && isset($value["name"]) && isset($value["path"])) {
                    $row["DSDOCUMENTOANEXO"] = $value["name"];
                    $row["DOCUMENTOANEXO"] = $value["path"];
                } else {
                    throw new \Exception("errorSavingFile", 1);
                }
                $dataSet = new DataSet($this->dataSourceName, array($row));
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                parent::save($newRequest, $response);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteFile(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->dataSourceName = $rows[0]{"TABLE"};
            foreach ($rows as $row) {
                $this->zhFileUploader->deleteFile($row["DOCUMENTOANEXO"]);
            }
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function downloadFile(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $file = $row["DOCUMENTOANEXOFULLPATH"];
        $name = $row["DSDOCUMENTOANEXO"];
        $nameArray = explode(".", $name);
        $type = end($nameArray);
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        header("Content-Disposition: attachment; filename=" . $name);
        readfile($file);
        exit;
    }
    
    public function apagaVigencia($row){
        try{
             $conditions = array(
                array(
                    "columnName" => 'CDFILIAL',
                    "operator" => "=",
                    "value" => $row['CDFILIAL']
                ),
                array(
                    "columnName" => "IDTIPORESTRICAO",
                    "operator" => "=",
                    "value" => $row['IDTIPORESTRICAO']
                ),
                array(
                    "columnName" => "DTVIGENCIA",
                    "operator" => "=",
                    "value" => $row['DTVIGENCIA']
                ),
                array(
                    "columnName" => "DTVIGENCIAFIM",
                    "operator" => "=",
                    "value" => $row['DTVIGENCIAFIM']
                )
            );
            $dataSet = $this->crudService->getFilteredDataset(strtolower($row['DataSourceDestino']), $conditions);
            $padraoPrato = $dataSet->getRows();
            $notOverlap = true;
            
            if(!empty($padraoPrato)){
                $sql = "DELETE FROM ".$row['DataSourceDestino']." WHERE CDFILIAL = :CDFILIAL AND IDTIPORESTRICAO = :IDTIPORESTRICAO AND DTVIGENCIA = :DTVIGENCIA AND DTVIGENCIAFIM = :DTVIGENCIAFIM";
                $query = $this->connection->prepare($sql);
                 
                $params = array(
                    "CDFILIAL"         => $row['CDFILIAL'],
                    "IDTIPORESTRICAO"  => $row['IDTIPORESTRICAO'],
                    "DTVIGENCIA"       => $row['DTVIGENCIA'],
                    "DTVIGENCIAFIM"    => $row['DTVIGENCIAFIM']
                ); 
                $query->execute($params);
            }else{
                $sql =  "SELECT 1 FROM " . $row['DataSourceDestino'] . 
                        " WHERE ((:DTVIGENCIA BETWEEN DTVIGENCIA AND DTVIGENCIAFIM) 
                             OR (:DTVIGENCIAFIM BETWEEN DTVIGENCIA AND DTVIGENCIAFIM)
                             OR (:DTVIGENCIA = DTVIGENCIA) )
                            AND (NRVIGENCIA <> :NRVIGENCIA OR :NRVIGENCIA IS NULL)
                            AND CDFILIAL= :CDFILIAL
                            AND IDTIPORESTRICAO = :IDTIPORESTRICAO
                            AND NRORG = :NRORG";
                $params = array(
                    "NRVIGENCIA"       => null,
                    "CDFILIAL"         => $row['CDFILIAL'],
                    "IDTIPORESTRICAO"  => $row['IDTIPORESTRICAO'],
                    "DTVIGENCIA"       => $row['DTVIGENCIA'],
                    "DTVIGENCIAFIM"    => $row['DTVIGENCIAFIM'],
                    "NRORG"            => $this->crudService->getEnvironment()->getNrOrg()
                );
                $dataSet = $this->connection->fetchAll($sql, $params);
                $notOverlap = empty($dataSet);
            }
            return !empty($padraoPrato) || $notOverlap;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function retornaServicosPorFilial($nrvigencia, $cdfilial, $tableName){
        $sql = "SELECT
            FIL.CDFILIAL,
            FIL.CDFILIAL || ' | ' || FIL.NMFILIAL AS NMFILIAL,
            S.CDSERVFILI,
            S.NMSERVICO AS NMSERVICO,
            TN.NR".$tableName."
        FROM
            ".$tableName."   TN
            JOIN SERVICO         S ON TN.CDSERVFILI = S.CDSERVFILI
                              AND TN.CDFILIAL = S.CDFILIAL
            JOIN FILIAL          FIL ON FIL.CDFILIAL = S.CDFILIAL
                               AND TN.CDFILIAL = FIL.CDFILIAL
        WHERE
            TN.NRVIGENCIA = :NRVIGENCIA
            AND TN.CDFILIAL = :CDFILIAL";
        $params = array(
            "NRVIGENCIA"    => $nrvigencia,
            "CDFILIAL"      => $cdfilial
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function retornaServicosRestrProd($nrvigencia, $cdfilial){
        $sql = "SELECT
            FIL.CDFILIAL,
            FIL.CDFILIAL || ' | ' || FIL.NMFILIAL AS NMFILIAL,
            S.CDSERVFILI,
            S.NMSERVICO AS NMSERVICO,
            RP.CDPRODUTO,
            RP.NRGRUPOPRATO,
            RP.NRRESTPRODGR,
            RP.NRGRUPOPRATOSERV
        FROM
            RESTPRODGR  RP
            JOIN SERVICO         S ON RP.CDSERVFILI = S.CDSERVFILI
                              AND RP.CDFILIAL = S.CDFILIAL
            JOIN FILIAL          FIL ON FIL.CDFILIAL = S.CDFILIAL
                               AND RP.CDFILIAL = FIL.CDFILIAL
        WHERE
            RP.NRVIGENCIA = :NRVIGENCIA
            AND RP.CDFILIAL = :CDFILIAL";
        $params = array(
            "NRVIGENCIA"    => $nrvigencia,
            "CDFILIAL"      => $cdfilial
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function retornaItemNr($row, $item, $tableName){
        if(isset($this->chaves[$item['NRGRUPOPRATOSERV']])){
            return $this->chaves[$item['NRGRUPOPRATOSERV']];
        }else{
            $sql = "SELECT GR.CDGRUPOPRATO,
                           GR.NMGRUPOPRATO,
                           GPS.NRGRUPOPRATOSERV
                      FROM GRUPOPRATOSERV GS 
                           INNER JOIN GRUPOPRATO GR ON GR.NRGRUPOPRATO = GS.NRGRUPOPRATO
                           LEFT JOIN GRUPOPRATO GD ON GD.CDGRUPOPRATO = GR.CDGRUPOPRATO AND GD.CDFILIAL = :CDFILIALDEST
                           LEFT JOIN GRUPOPRATOSERV GPS ON GPS.CDSERVFILI = GS.CDSERVFILI AND GPS.CDSERVFILI = :CDSERVFILI AND GPS.NRGRUPOPRATO = GD.NRGRUPOPRATO
                     WHERE GR.CDFILIAL = :CDFILIALORIG
                       AND GS.CDSERVFILI = :CDSERVFILI
                       AND EXISTS (SELECT 1 FROM ".$tableName." IT WHERE IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                                                     AND IT.NRGRUPOPRATOSERV = :NRGRUPOPRATOSERVORIG
                                                                     AND IT.NRGRUPOPRATO = GR.NRGRUPOPRATO)";
            $params = array(
                "CDFILIALDEST"            => $row['CDFILIAL'],
                "CDFILIALORIG"            => $row['CDFILIALORIG'],
                "CDSERVFILI"              => $item['CDSERVFILI'],
                "NRGRUPOPRATOSERVORIG"    => $item['NRGRUPOPRATOSERV']
            );
            $dataSet = $this->connection->fetchAll($sql, $params);
            if(empty($dataSet)){
                return null;
            }else{
                $this->chaves[$item['NRGRUPOPRATOSERV']] = $dataSet[0];
                return $dataSet[0];
            }
        }
    }

    private function queryUnidadeVigencia($tableName){
        $sql = <<<SQL
            SELECT 
                V.CDFILIAL, 
                F.NMFILIAL,
                V.NRVIGENCIA, 
                TO_CHAR(DTVIGENCIA, 'MM/YYYY') VIGENCIA_INICIAL,
                TO_CHAR(DTVIGENCIAFIM, 'MM/YYYY') VIGENCIA_FINAL,
                V.CDFILIAL|| ' | ' || TO_CHAR(DTVIGENCIA, 'MM/YYYY') || 
                CASE WHEN DTVIGENCIAFIM IS NULL 
                THEN 
                    '' 
                ELSE 
                    ' | ' || TO_CHAR(DTVIGENCIAFIM, 'MM/YYYY') 
                END  DESCRIPTION
            FROM
                $tableName V INNER JOIN FILIAL F ON F.CDFILIAL = V.CDFILIAL
                                   INNER JOIN FILIOPER FO ON FO.CDFILIAL = V.CDFILIAL AND FO.CDOPERADOR = :P_CDOPERADOR
            WHERE V.NRORG = :P_NRORG
            AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
            ORDER BY V.CDFILIAL, V.DTVIGENCIA
SQL;

        return $this->connection->prepare($sql);
    }  
    
    public function getUnitVig(DTO\Request\Filter $request, DTO\Response $response){
        //var_dump remover default values
        $conditions = $request->getFilterCriteria()->getConditions();
        $screenCode   = Utilities::getValueInFilterDataSourceByName($conditions, 'SCREEN_CODE');
        $idTipoRestricao = Utilities::getValueInFilterDataSourceByName($conditions, 'IDTIPORESTRICAO');
        $tables = self::TABLES_BY_SCREEN_CODE[$screenCode] ?? null;
        if($screenCode && $tables){
            $stmt = $this->queryUnidadeVigencia($tables["TABLE_VIG"]);
            $stmt->execute(array(
                'P_NRORG' => $this->environment->getNrOrgTrab(),
                'P_CDOPERADOR' => $this->environment->getCdOperador(),
                'P_IDTIPORESTRICAO' => $idTipoRestricao
            ));
            $response->addDataSet(new DataSet('unidadeVigencia', $stmt->fetchAll()));
        }
    }

    private function queryServiceByUnit($tableName){
        $sql = <<<SQL
        SELECT  S.CDFILIAL, 
                F.NMFILIAL,
                S.CDSERVFILI, 
                S.NMSERVICO, 
                S.CDFILIAL||S.CDSERVFILI AS CHAVE
            FROM SERVICO S 
                JOIN FILIAL F 
                ON S.CDFILIAL = F.CDFILIAL
            WHERE   S.IDSERVATIV = 'S' AND 
                    S.IDTIPOSERV = 'N' AND
                    
                    S.CDFILIAL IN (
                        SELECT DISTINCT V.CDFILIAL
                            FROM $tableName V 
                                INNER JOIN FILIOPER FO 
                                    ON FO.CDFILIAL = V.CDFILIAL 
                                    AND FO.CDOPERADOR = :P_CDOPERADOR
                            WHERE V.NRORG = :P_NRORG
                                AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                                AND (
                                    INSTR(:P_NRVIGENCIA,'_'|| V.NRVIGENCIA ||'_') <> 0 
                                    OR (:P_TNRVIGENCIA = 'T')
                                )
                    )
            ORDER BY S.CDFILIAL, S.CDSERVFILI 
SQL;
        return $this->connection->prepare($sql);
    }

    public function getServiceByUnit(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $screenCode  = Utilities::getValueInFilterDataSourceByName($conditions, 'SCREEN_CODE');
        $vigencias   = Utilities::getValueInFilterDataSourceByName($conditions, 'NRVIGENCIA');
        $tVigencia   = Utilities::getValueInFilterDataSourceByName($conditions, 'TNRVIGENCIA');
        $idTipoRestricao   = Utilities::getValueInFilterDataSourceByName($conditions, 'IDTIPORESTRICAO');
        $cdOperador = $this->environment->getCdOperador();
        $nrOrg = $this->environment->getNrOrg();
        $tables = self::TABLES_BY_SCREEN_CODE[$screenCode] ?? null;
        $stmt = $this->queryServiceByUnit($tables["TABLE_VIG"]);
        if($tVigencia == "T"){
            $vigencias = "";
        }
        $stmt->execute(array(
            'P_CDOPERADOR' => $cdOperador,
            'P_NRORG'      => $nrOrg,
            'P_IDTIPORESTRICAO' => $idTipoRestricao,
            'P_NRVIGENCIA' => $vigencias,
            'P_TNRVIGENCIA' => $tVigencia
        ));
        $response->addDataSet(new DataSet('servico', $stmt->fetchAll()));
    }

    private function queryFiliServVig($tableName){
        $sql = <<<SQL
        SELECT 
            V.CDFILIAL||S.CDSERVFILI||V.NRVIGENCIA AS CHAVE
        FROM $tableName V INNER JOIN SERVICO S   
                                    ON  V.CDFILIAL    = S.CDFILIAL 
                                INNER JOIN FILIOPER FO 
                                    ON  FO.CDFILIAL   = V.CDFILIAL 
                                    AND FO.CDOPERADOR = :P_CDOPERADOR
        WHERE 
            V.NRORG = :P_NRORG AND 
            V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND
            ( 
                INSTR(:P_NRVIGENCIA,'_'||V.NRVIGENCIA ||'_') <> 0 
                OR (:P_TNRVIGENCIA = 'T')
            ) AND
            (
                INSTR(:P_FILISERV,S.CDFILIAL||S.CDSERVFILI) <> 0 
                OR (:P_FILISERV = 'T')
            ) AND
            (V.CDFILIAL||S.CDSERVFILI||V.NRVIGENCIA <> :P_CURRENT_SERVICE) AND
            S.IDSERVATIV = 'S' AND 
            S.IDTIPOSERV = 'N'
        ORDER BY S.CDFILIAL, S.CDSERVFILI
SQL;
        return $this->connection->prepare($sql);
    }

    private function queryNameUnitServ($servicesConcat){
        $sql = <<<SQL
            SELECT
                F.CDFILIAL,
                F.NMFILIAL,
                S.CDSERVFILI,
                S.NMSERVICO
            FROM
                FILIAL    F
                LEFT JOIN SERVICO   S ON S.CDFILIAL = F.CDFILIAL
                                    AND F.NRORG = :P_NRORG
            WHERE 
                F.CDFILIAL = :P_CDFILIAL
                AND S.CDSERVFILI IN ($servicesConcat)
SQL;
        return $this->connection->prepare($sql);
    }

    private function getNameFiliServ($filiServ){
        $arrNames = array();
        $nrorg = $this->environment->getNrOrg();
        $params = array(
            'P_NRORG' => $nrorg
        );
        foreach($filiServ as $unit => $servicesConcat){
            $stmt = $this->queryNameUnitServ($servicesConcat);
            $params['P_CDFILIAL'] = $unit;
            $stmt->execute($params);
            $filiServNames = $stmt->fetchAll();
            foreach($filiServNames as $row) {
                $cdfilial = $row['CDFILIAL'];
                $cdservfili = $row['CDSERVFILI'];
                if(!isset($arrNames[$cdfilial])){
                    $arrNames[$cdfilial] = [ 'SERVICES' => [] ];
                }
                $arrNames[$cdfilial]['NMFILIAL'] = $row['NMFILIAL'];
                $arrNames[$cdfilial]['SERVICES'][$cdservfili] = $row['NMSERVICO'];
            }
        }
        return $arrNames;
    }

    private function getFiliServErros($erros){ // retornar servicos separados por virgula pra consultar no banco respectivo nomes;
        $filiServ = array();
        foreach($erros as $item){
            $unit = $item["PARAMS"]["PARAM0"] ?? null;
            $service = $item["PARAMS"]["PARAM1"] ?? null;
            if($unit && $service){
                if(isset($filiServ[$unit])){
                    $filiServ[$unit][$service] = ""; 
                } else {
                    $filiServ[$unit] = array($service => "");
                }
            }
        }
        $units = array_keys($filiServ); 
        $result = array();
        foreach($filiServ as $unit => $services){
            $services = array_keys($services);
            $services = array_map(function($service){
                return "'$service'";
            }, $services);
            $concat = implode(", ", $services );
            $result[$unit] = $concat;
        }
        return $result;
    }

    public function exportItems(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $isAllPeriods = ($row['TNRVIGENCIA'] ?? 'N') == 'S';
        $isAllServices = ($row['TCDSERVFILI'] ?? 'N') == 'S';
        $screenCode = $row['SCREEN_CODE'] ?? null;
        $tables = self::TABLES_BY_SCREEN_CODE[$screenCode] ?? null;
        $currentService = $row['CDFILIAL'] . $row['CDSERVFILI'] . $row['NRVIGENCIA_ORIGEM'];
        $params = array(
            'P_CDOPERADOR'      => $this->environment->getCdOperador(),
            'P_NRORG'           => $this->environment->getNrOrg(),
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_NRVIGENCIA'      => '',
            'P_TNRVIGENCIA'     => 'N',
            'P_FILISERV'        => 'N',
            'P_CURRENT_SERVICE' => $currentService
        );
        $params['P_TNRVIGENCIA'] = $isAllPeriods  ? 'T' : 'N';
        $params['P_NRVIGENCIA']  = $isAllPeriods  ? ''  : $row['NRVIGENCIA'];
        $params['P_FILISERV']    = $isAllServices ? 'T' : $row['FILISERV'];
        
        $stmt = $this->queryFiliServVig($tables['TABLE_VIG']);
        $stmt->execute($params);

        $params['P_TABELA'] = $tables['TABLE_VIG'];
        $params['P_CDFILIAL'] = $row['CDFILIAL'];
        $params['P_IDOPCAO']  = $row['IDOPCAO'];
        $filiServVig = $stmt->fetchAll();
        $filiServVig = array_column($filiServVig, 'CHAVE');
        
        $nrlista = $this->util->insertLista($filiServVig);
        $nrlistaItems = $this->util->insertLista($row['LIST_ITEMS']);
        $params['P_NRLISTA'] = $nrlistaItems;
        $params['P_NRLISTA_DEST'] = $nrlista;
        $params['P_CDSERVFILI'] = $row['CDSERVFILI'];
        $params['P_NRVIGENCIA_ORIGEM'] = $row['NRVIGENCIA_ORIGEM'];
        try {
            $result = $this->exportVigProcedure($params, $response);
            if(is_array($result['LISTAERROS']) && !empty($result['LISTAERROS'])){
                $response->addDataSet(new DataSet("LISTAERROS", $result['LISTAERROS']));
                $filiServ = $this->getFiliServErros($result['LISTAERROS']);
                $filiServNames = $this->getNameFiliServ($filiServ);
                $response->addDataSet(new DataSet("FILISERV", $filiServNames));
            }
        } catch(\Exception $e){
            throw $e;
        } finally {
            if($nrlista) $this->util->deleteLista($nrlista);
            if($nrlistaItems) $this->util->deleteLista($nrlistaItems);
        }
    }
    
    public function exportVigProcedure ($params, $response) {
        $procedure = new StoredProcedure($this->connection, 'EXPORTA_VIG_ITEM_OTIMIZADOR');
        $procedure->addParam(new Param('P_TABELA',            Param::PARAM_INPUT,  $params['P_TABELA']));
        $procedure->addParam(new Param('P_NRVIGENCIA',        Param::PARAM_INPUT,  $params['P_NRVIGENCIA_ORIGEM']));
        $procedure->addParam(new Param('P_CDFILIAL',          Param::PARAM_INPUT,  $params['P_CDFILIAL']));
        $procedure->addParam(new Param('P_CDSERVFILI',        Param::PARAM_INPUT,  $params['P_CDSERVFILI']));      
        $procedure->addParam(new Param('P_IDTIPORESTRICAO',   Param::PARAM_INPUT,  $params['P_IDTIPORESTRICAO']));      
        $procedure->addParam(new Param('P_NRLISTA_DEST',      Param::PARAM_INPUT,  $params['P_NRLISTA_DEST']));      
        $procedure->addParam(new Param('P_NRLISTA',           Param::PARAM_INPUT,  $params['P_NRLISTA']));      
        $procedure->addParam(new Param('P_IDOPCAO',           Param::PARAM_INPUT,  $params['P_IDOPCAO']));      
        $procedure->addParam(new Param('P_CDOPERADOR',        Param::PARAM_INPUT,  $this->environment->getCdOperador()));
        $procedure->addParam(new Param('P_LISTA_ERROS',       Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));      
        
        $responseProcedure = $procedure->execute();
        $nrlistaErros = $responseProcedure['P_LISTA_ERROS'] ?? null;
        $result = array(
            "LISTAERROS" => array()
        );
        try {
            if($nrlistaErros){
                $messages = $this->util->getMessageErrorList($nrlistaErros);
                $messages = array_column($messages, "DSERRO");
                $convertedMessages = array_map('PLA\Util\Util::converter', $messages);
                $result['LISTAERROS'] = $convertedMessages;
            }
        } catch(\Exception $e){
            throw $e;
        } finally {
            if($nrlistaErros){
                $this->util->deleteListaErros($nrlistaErros);
            }
        }
        return $result;
    }
}